{% load i18n %}
<div class="well well-small">
    <h3>{% blocktrans %}Image Registry tool:{% endblocktrans %}</h3>
    <br />
    <p>
        {% blocktrans %}Image Registry is used to provide additional information about images for Data Processing.{% endblocktrans %}
    </p>
    <p>
        {% blocktrans %}Specified User Name will be used by Data Processing to apply configs and manage processes on instances.{% endblocktrans %}
    </p>

        <div class="bs-component">
        <div class="alert alert-dismissable alert-warning">
          <button type="button" class="close" data-dismiss="alert">×</button>
          <h4><span class="ng-scope">{% blocktrans %}Warning!{% endblocktrans %}</span></h4>
          <p> {% blocktrans trimmed %} Input correct User Name for image. For Ubuntu images input 'ubuntu', 'fedora' for Fedora images,
               'cloud-user' for CentOS 6.x images and 'centos' for CentOS 7.x images.{% endblocktrans %} </p>
        </div>
      </div>
    <p>
        {% blocktrans trimmed %}Tags are used for filtering images suitable for each plugin and each Data Processing version.
        To add required tags, select a plugin and a Data Processing version and click the &quot;Add&quot; button.{% endblocktrans %}
    </p>
    <p>
        {% blocktrans %}You may also add any custom tag.{% endblocktrans %}
    </p>
    <p>
        {% blocktrans %}Unnecessary tags may be removed by clicking a cross near tag's name.{% endblocktrans %}
    </p>
</div>
