{% load i18n %}

<h4>{% trans "Cluster health checks" %}</h4>
<table id="sahara_health_checks_table" class="table table-bordered datatable">
    <thead>
        <tr>
            <th>{% trans "Status" %}</th>
            <th>{% trans "Name" %}</th>
            <th>{% trans "Duration" %}</th>
            <th>{% trans "Description" %}</th>
        </tr>
    </thead>
    <tbody id="sahara_health_checks_body">
    </tbody>
</table>

<script type="text/javascript">
    var data_update_url = "{{ data_update_url }}"
    $(function() {
        // Initialize.
        horizon.verifications.data_update_url = "{{ data_update_url }}"
        horizon.verifications.update_health_checks();
    });
</script>
