{% load i18n horizon %}
<div class="well">
<p>
  {% blocktrans %}Create a Data Source with a specified name.{% endblocktrans %}
</p>
<p>
  {% blocktrans %}Select the type of your Data Source.{% endblocktrans %}
</p>
<p>
  {% blocktrans %}You may need to enter the username and password for your Data Source.{% endblocktrans %}
</p>
<p>
  {% blocktrans %}For Data Sources on a Manila share, choose the share and enter the path relative to the share (example: /outputdir/myinputfile.txt){% endblocktrans %}
</p>
<p>
  {% blocktrans %}For Data Sources in S3 storage, the URL should be specified in the format s3a://bucket/path. Also, each of the access properties may be optional if you have configured that property on your cluster manually.{% endblocktrans %}
</p>

<p>
  {% blocktrans %}You may also enter an optional description for your Data Source.{% endblocktrans %}
</p>
</div>
