{% load i18n horizon %}
<div class="well">
<p>
  {% blocktrans trimmed %}<b>Important</b>:  The name that you give your job binary will be the name used in your job execution.
  If your binary requires a particular name or extension (ie: ".jar"), be sure to include it here.{% endblocktrans %}
</p>
<p>
  {% blocktrans %}Select the storage type for your job binary.{% endblocktrans %}
  <ul class="list-bullet">
    <li>{% blocktrans %}Data Processing internal database{% endblocktrans %}</li>
    <li>{% blocktrans %}Swift{% endblocktrans %}</li>
    <li>{% blocktrans %}S3{% endblocktrans %}</li>
  </ul>
</p>
<p>
  {% blocktrans %}For Data Processing internal job binaries, you may choose from the following:{% endblocktrans %}
  <ul class="list-bullet">
    <li>{% blocktrans %}Choose an existing file{% endblocktrans %}</li>
    <li>{% blocktrans %}Upload a new file{% endblocktrans %}</li>
    <li>{% blocktrans %}Create a script to be uploaded dynamically{% endblocktrans %}</ul>
  </ul>
</p>
<p>
  {% blocktrans %}For Object Store job binaries, you must:{% endblocktrans %}
  <ul class="list-bullet">
    <li>{% blocktrans %}Enter the URL for the file{% endblocktrans %}</li>
    <li>{% blocktrans %}Enter the username and password required to access that file{% endblocktrans %}</li>
  </ul>
</p>
<p>
  {% blocktrans %}For S3 job binaries, you must:{% endblocktrans %}
  <ul class="list-bullet">
    <li>{% blocktrans %}Enter the URL for the file{% endblocktrans %}</li>
    <li>{% blocktrans %}Enter the access key and secret key required to access that file{% endblocktrans %}</li>
    <li>{% blocktrans %}Enter the endpoint of the S3 service where the file is stored{% endblocktrans %}</li>
  </ul>
</p>
<p>
  {% blocktrans %}You may also enter an optional description for your job binary.{% endblocktrans %}
</p>
</div>
